use rand::Rng;
use crate::patterns::{Pattern, PixelColor};
use crate::Strip;
use serde::{Serialize, Deserialize};

/// # rain pattern
/// randomly fall colors from top or bottom.
///
///
/// # note
///
/// `stability` is the probability (between 0 an 1) that any led will light on.
/// The probability of at least one led light on is `stability*number of lines`
///
/// this pattern works only for 5x5 square
#[derive(Serialize, Deserialize, Debug, Eq, Hash, PartialEq, Copy, Clone)]
pub struct Rain<const N: usize> {
    pub(crate) color: PixelColor,
    pub(crate) background_color: Option<PixelColor>,
    pub(crate) max_iteration: Option<usize>,
    #[serde(default)]
    pub(crate) current_iteration: usize,
    pub(crate) stability: usize,
    pub(crate) lines: usize,
    #[serde(skip)]
    drops : Strip<N>,
}

impl<const N: usize> Pattern for Rain<N> {
    type Strip = Strip<N>;

    fn init(&mut self) -> Option<Self::Strip> {
        if let Some(color) = self.background_color {
            self.drops = Strip::<N>::new(color);
            Some(Strip::<N>::new(color))
        } else {
            None
        }
    }

    fn is_last_iteration(&self) -> bool {
        match self.max_iteration {
            None => { false }
            Some(max_iter) => {
                if self.current_iteration >= max_iter {
                    true
                } else {
                    false
                }
            }
        }
    }
}

impl<const N: usize> Iterator for Rain<N> {
    type Item = Strip<N>;

    fn next(&mut self) -> Option<Self::Item> {
        let mut strip = Strip::<N>::default();

        for i in 0..(N-self.lines) {
            strip[i+self.lines] = self.drops[i];
        }
        let mut rng = rand::thread_rng();
        for i in 0..self.lines {
            if rng.gen_bool(self.stability as f64 / 100 as f64) {
                strip[i] = Some(self.color);
            } else {
                let c = strip[i+self.lines];
                if self.background_color != c && c != Some(PixelColor::default()) {
                    strip[i] = Some(strip[i+self.lines].unwrap()/ 2);
                } else {
                    strip[i] = self.background_color;
                }
            }
        }
        self.drops = strip.clone();

        self.current_iteration += 1;
        Some(strip)
    }
}